@@ -23,6 +23,7 @@ from toolchest_client.api.output import Output
 from toolchest_client.api.urls import get_pipeline_segment_instances_url
 from toolchest_client.files import OutputType, path_is_s3_uri, path_is_http_url
 from .status import Status, ThreadStatus
+from ..files.s3 import UploadTracker
 
 
 class Query:
@@ -37,9 +38,13 @@ class Query:
     WAIT_FOR_JOB_DELAY = 1
     # Multiple of seconds used when pretty printing job status to output.
     PRINTED_TIME_INTERVAL = 5
+    # Max number of retries on status check timeouts.
+    RETRY_STATUS_CHECK_LIMIT = 5
 
     def __init__(self, stored_output=None, is_async=False, pipeline_segment_instance_id=None):
-        self.HEADERS = dict()
+        # Configure Toolchest API authorization.
+        self.HEADERS = get_headers()
+
         if pipeline_segment_instance_id:
             self.PIPELINE_SEGMENT_INSTANCE_ID = pipeline_segment_instance_id
             self.PIPELINE_SEGMENT_INSTANCE_URL = "/".join([
@@ -50,22 +55,26 @@ class Query:
                 self.PIPELINE_SEGMENT_INSTANCE_URL,
                 "status",
             ])
+        else:
+            self.PIPELINE_SEGMENT_INSTANCE_ID = None
+            self.PIPELINE_SEGMENT_INSTANCE_URL = None
+            self.STATUS_URL = None
 
         self.mark_as_failed = False
         self.thread_name = ''
         self.thread_statuses = None
         self.is_async = is_async
 
+        self.status_check_retries = 0
+
         self.unpacked_output_paths = None
         self.output = stored_output if stored_output else Output()
 
-        # Configure Toolchest API authorization.
-        self.HEADERS = get_headers()
-
     def run_query(self, tool_name, tool_version, input_prefix_mapping,
                   output_type, tool_args=None, database_name=None, database_version=None,
-                  custom_database_path=None, output_name="output", output_primary_name=None,
-                  input_files=None, output_path=None, skip_decompression=False, thread_statuses=None):
+                  custom_database_path=None, input_files=None, is_database_update=False,
+                  output_path=None, output_primary_name=None, skip_decompression=False,
+                  thread_statuses=None):
         """Executes a query to the Toolchest API.
 
         :param tool_name: Tool to be used.
@@ -74,13 +83,13 @@ class Query:
         :param database_name: Name of database to be used.
         :param database_version: Version of database to be used.
         :param custom_database_path: Path (S3 URI) to a custom database.
-        :param input_prefix_mapping: Mapping of input filepaths to associated prefix tags (e.g., "-1")
-        :param output_name: (optional) Internal name of file outputted by the tool.
+        :param input_prefix_mapping: Mapping of input filepaths to associated prefix tags (e.g., "-1").
+        :param is_database_update: Whether the call is to update an existing database.
         :param output_primary_name: (optional) basename of the primary output (e.g. "sample.fastq").
         :param input_files: List of paths to be passed in as input.
         :param output_path: Path (client-side) where the output file will be downloaded.
-        :param output_type: Type (e.g. GZ_TAR) of the output file
-        :param skip_decompression: Whether to skip decompression of the output file, if it is an archive
+        :param output_type: Type (e.g. GZ_TAR) of the output file.
+        :param skip_decompression: Whether to skip decompression of the output file, if it is an archive.
         :param thread_statuses: Statuses of all threads, shared between threads.
         """
         self.thread_name = threading.current_thread().getName()
@@ -94,16 +103,17 @@ class Query:
             database_name=database_name,
             database_version=database_version,
             custom_database_path=custom_database_path,
-            output_name=output_name,
-            output_primary_name=output_primary_name,
+            is_database_update=is_database_update,
             tool_name=tool_name,
             tool_version=tool_version,
             tool_args=tool_args,
             output_file_path=output_path,
+            output_primary_name=output_primary_name,
         )
         create_content = create_response.json()
 
         self._update_thread_status(ThreadStatus.INITIALIZED)
+        self.mark_as_failed = True
 
         self.PIPELINE_SEGMENT_INSTANCE_ID = create_content["id"]
         self.PIPELINE_SEGMENT_INSTANCE_URL = "/".join([
@@ -116,7 +126,10 @@ class Query:
         ])
 
         self.output.set_run_id(self.PIPELINE_SEGMENT_INSTANCE_ID)
-        self.mark_as_failed = True
+        self.output.set_database(
+            database_name=create_content.get("database_name"),
+            database_version=create_content.get("database_version"),
+        )
 
         self._check_if_should_terminate()
         self._update_thread_status(ThreadStatus.UPLOADING)
@@ -142,8 +155,8 @@ class Query:
 
     def _send_initial_request(self, tool_name, tool_version, tool_args,
                               database_name, database_version, custom_database_path,
-                              output_name, output_primary_name, output_file_path,
-                              compress_output):
+                              output_primary_name, output_file_path, compress_output,
+                              is_database_update):
         """Sends the initial request to the Toolchest API to create the query.
 
         Returns the response from the POST request.
@@ -155,11 +168,12 @@ class Query:
             "custom_database_s3_location": custom_database_path,
             "database_name": database_name,
             "database_version": database_version,
-            "output_file_name": output_name,
-            "output_file_primary_name": output_primary_name,
+            "is_database_update": is_database_update,
             "tool_name": tool_name,
             "tool_version": tool_version,
             "output_file_path": output_file_path,
+            "output_file_primary_name": output_primary_name,
+
         }
 
         create_response = requests.post(
@@ -268,6 +282,7 @@ class Query:
                         file_path,
                         input_file_keys["bucket"],
                         input_file_keys["object_name"],
+                        Callback=UploadTracker(file_path)
                     )
                     self._update_file_size(input_file_keys["file_id"])
                 except Exception as e:
@@ -316,11 +331,13 @@ class Query:
         self._update_thread_status(ThreadStatus.FAILED)
 
         # Mark pipeline segment instance as failed
-        requests.put(
-            self.STATUS_URL,
-            headers=self.HEADERS,
-            json={"status": Status.FAILED, "error_message": error_message},
-        )
+        if self.STATUS_URL:
+            requests.put(
+                self.STATUS_URL,
+                headers=self.HEADERS,
+                json={"status": Status.FAILED, "error_message": error_message},
+            )
+
         self.mark_as_failed = False
         if force_raise:
             raise ToolchestException(error_message) from None
@@ -352,10 +369,15 @@ class Query:
         start_time = time.time()
         while status != Status.READY_TO_TRANSFER_TO_CLIENT:
             self._check_if_should_terminate()
-            status_response = self.get_job_status(return_error=True)
-            status = status_response['status']
-            if status == Status.FAILED:
-                raise ToolchestJobError(status_response['error_message'])
+            try:
+                status_response = self.get_job_status(return_error=True)
+                status = status_response['status']
+                if status == Status.FAILED:
+                    raise ToolchestJobError(status_response['error_message'])
+            except TimeoutError as err:
+                self.status_check_retries += 1
+                if self.status_check_retries > self.RETRY_STATUS_CHECK_LIMIT:
+                    raise ToolchestJobError("Status check timed out during execution, retry limit exceeded.") from err
 
             elapsed_time = time.time() - start_time
             leftover_delay = elapsed_time % self.WAIT_FOR_JOB_DELAY
